@@ -56,29 +56,6 @@ DllMain (HINSTANCE hinstDLL,
   return TRUE;
 }
 
-static inline gboolean
-pid_is_running (gint pid)
-{
-  HANDLE h;
-  DWORD exitcode = 0;
-
-  h = OpenProcess (PROCESS_QUERY_INFORMATION, FALSE, pid);
-  GetExitCodeProcess (h, &exitcode);
-  CloseHandle (h);
-
-  return exitcode == STILL_ACTIVE;
-}
-
-#else
-
-#include <sys/types.h>
-#include <signal.h>
-
-static inline gboolean
-pid_is_running (gint pid)
-{
-  return (kill (pid, 0) == 0);
-}
 #endif
 
 
@@ -99,12 +76,15 @@ guint gegl_debug_flags = 0;
 #include "operation/gegl-operation-handlers-private.h"
 #include "buffer/gegl-buffer-private.h"
 #include "buffer/gegl-buffer-iterator-private.h"
+#include "buffer/gegl-buffer-swap-private.h"
+#include "buffer/gegl-compression.h"
 #include "buffer/gegl-tile-backend-ram.h"
 #include "buffer/gegl-tile-backend-file.h"
 #include "gegl-config.h"
 #include "gegl-stats.h"
 #include "graph/gegl-node-private.h"
 #include "gegl-random-private.h"
+#include "gegl-parallel-private.h"
 
 static gboolean  gegl_post_parse_hook (GOptionContext *context,
                                        GOptionGroup   *group,
@@ -120,58 +100,6 @@ static GeglModuleDB *module_db   = NULL;
 
 static glong         global_time = 0;
 
-static gboolean      swap_init_done = FALSE;
-
-static gchar        *swap_dir = NULL;
-
-static void
-gegl_init_swap_dir (void)
-{
-  gchar *swapdir = NULL;
-
-  if (config->swap)
-    {
-      if (g_ascii_strcasecmp (config->swap, "ram") == 0)
-        {
-          swapdir = NULL;
-        }
-      else
-        {
-          swapdir = g_strstrip (g_strdup (config->swap));
-
-          /* Remove any trailing separator, unless the path is only made of a leading separator. */
-          while (strlen (swapdir) > strlen (G_DIR_SEPARATOR_S) && g_str_has_suffix (swapdir, G_DIR_SEPARATOR_S))
-            swapdir[strlen (swapdir) - strlen (G_DIR_SEPARATOR_S)] = '\0';
-        }
-    }
-
-  if (swapdir &&
-      ! g_file_test (swapdir, G_FILE_TEST_IS_DIR) &&
-      g_mkdir_with_parents (swapdir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
-    {
-      g_clear_pointer (&swapdir, g_free);
-    }
-
-  g_object_set (config, "swap", swapdir, NULL);
-
-  swap_dir = swapdir;
-
-  swap_init_done = TRUE;
-}
-
-/* Return the swap directory, or NULL if swapping is disabled */
-const gchar *
-gegl_swap_dir (void)
-{
-  if (!swap_init_done)
-  {
-    g_critical ("swap parsing not done");
-    gegl_init_swap_dir ();
-  }
-
-  return swap_dir;
-}
-
 static void load_module_path(gchar *path, GeglModuleDB *db);
 
 static void
@@ -269,13 +197,14 @@ gegl_init (gint    *argc,
   g_option_context_free (context);
 }
 
-static gchar    *cmd_gegl_swap           = NULL;
-static gchar    *cmd_gegl_cache_size     = NULL;
-static gchar    *cmd_gegl_chunk_size     = NULL;
-static gchar    *cmd_gegl_quality        = NULL;
-static gchar    *cmd_gegl_tile_size      = NULL;
-static gchar    *cmd_gegl_threads        = NULL;
-static gboolean *cmd_gegl_disable_opencl = NULL;
+static gchar    *cmd_gegl_swap             = NULL;
+static gchar    *cmd_gegl_swap_compression = NULL;
+static gchar    *cmd_gegl_cache_size       = NULL;
+static gchar    *cmd_gegl_chunk_size       = NULL;
+static gchar    *cmd_gegl_quality          = NULL;
+static gchar    *cmd_gegl_tile_size        = NULL;
+static gchar    *cmd_gegl_threads          = NULL;
+static gboolean *cmd_gegl_disable_opencl   = NULL;
 
 static const GOptionEntry cmd_entries[]=
 {
@@ -284,6 +213,11 @@ static const GOptionEntry cmd_entries[]=
      G_OPTION_ARG_STRING, &cmd_gegl_swap,
      N_("Where GEGL stores its swap"), "<uri>"
     },
+    {
+     "gegl-swap-compression", 0, 0,
+     G_OPTION_ARG_STRING, &cmd_gegl_swap_compression,
+     N_("Compression algorithm used for data stored in the swap"), "<algorithm>"
+    },
     {
      "gegl-cache-size", 0, 0,
      G_OPTION_ARG_STRING, &cmd_gegl_cache_size,
@@ -333,21 +267,20 @@ gegl_get_option_group (void)
   return group;
 }
 
-static void gegl_config_set_defaults (GeglConfig *config)
-{
-  gchar *swapdir = g_build_filename (g_get_user_cache_dir(),
-                                     GEGL_LIBRARY,
-                                     "swap",
-                                     NULL);
-  g_object_set (config,
-                "swap", swapdir,
-                NULL);
-
-  g_free (swapdir);
-}
-
 static void gegl_config_parse_env (GeglConfig *config)
 {
+  if (g_getenv ("GEGL_MIPMAP_RENDERING"))
+    {
+      const gchar *value = g_getenv ("GEGL_MIPMAP_RENDERING");
+      if (!strcmp (value, "1")||
+          !strcmp (value, "true")||
+          !strcmp (value, "yes"))
+        g_object_set (config, "mipmap-rendering", TRUE, NULL);
+      else
+        g_object_set (config, "mipmap-rendering", TRUE, NULL);
+    }
+
+
   if (g_getenv ("GEGL_QUALITY"))
     {
       const gchar *quality = g_getenv ("GEGL_QUALITY");
@@ -363,7 +296,12 @@ static void gegl_config_parse_env (GeglConfig *config)
     }
 
   if (g_getenv ("GEGL_CACHE_SIZE"))
-    config->tile_cache_size = atoll(g_getenv("GEGL_CACHE_SIZE"))* 1024*1024;
+    {
+      g_object_set (config,
+                    "tile-cache-size",
+                    (guint64) atoll(g_getenv("GEGL_CACHE_SIZE")) * 1024 * 1024,
+                    NULL);
+    }
 
   if (g_getenv ("GEGL_CHUNK_SIZE"))
     config->chunk_size = atoi(g_getenv("GEGL_CHUNK_SIZE"));
@@ -371,10 +309,16 @@ static void gegl_config_parse_env (GeglConfig *config)
   if (g_getenv ("GEGL_TILE_SIZE"))
     {
       const gchar *str = g_getenv ("GEGL_TILE_SIZE");
-      config->tile_width = atoi(str);
+      gint         width;
+      gint         height;
+      width = height = atoi(str);
       str = strchr (str, 'x');
       if (str)
-        config->tile_height = atoi(str+1);
+        height = atoi(str+1);
+      g_object_set (config,
+                    "tile-width",  width,
+                    "tile-height", height,
+                    NULL);
     }
 
   if (g_getenv ("GEGL_THREADS"))
@@ -409,15 +353,20 @@ static void gegl_config_parse_env (GeglConfig *config)
 
   if (g_getenv ("GEGL_SWAP"))
     g_object_set (config, "swap", g_getenv ("GEGL_SWAP"), NULL);
+
+  if (g_getenv ("GEGL_SWAP_COMPRESSION"))
+    {
+      g_object_set (config,
+                    "swap-compression", g_getenv ("GEGL_SWAP_COMPRESSION"),
+                    NULL);
+    }
 }
 
 GeglConfig *gegl_config (void)
 {
   if (!config)
-    {
-      config = g_object_new (GEGL_TYPE_CONFIG, NULL);
-      gegl_config_set_defaults (config);
-    }
+    config = g_object_new (GEGL_TYPE_CONFIG, NULL);
+
   return config;
 }
 
@@ -434,43 +383,6 @@ void gegl_reset_stats (void)
   gegl_stats_reset (gegl_stats ());
 }
 
-static void swap_clean (void)
-{
-  const gchar  *swap_dir = gegl_swap_dir ();
-  GDir         *dir;
-
-  if (! swap_dir)
-    return;
-
-  dir = g_dir_open (gegl_swap_dir (), 0, NULL);
-
-  if (dir != NULL)
-    {
-      GPatternSpec *pattern = g_pattern_spec_new ("*");
-      const gchar  *name;
-
-      while ((name = g_dir_read_name (dir)) != NULL)
-        {
-          if (g_pattern_match_string (pattern, name))
-            {
-              gint readpid = atoi (name);
-
-              if (!pid_is_running (readpid))
-                {
-                  gchar *fname = g_build_filename (gegl_swap_dir (),
-                                                   name,
-                                                   NULL);
-                  g_unlink (fname);
-                  g_free (fname);
-                }
-            }
-         }
-
-      g_pattern_spec_free (pattern);
-      g_dir_close (dir);
-    }
-}
-
 void gegl_temp_buffer_free (void);
 
 void
@@ -488,7 +400,10 @@ gegl_exit (void)
   gegl_tile_cache_destroy ();
   gegl_operation_gtype_cleanup ();
   gegl_operation_handlers_cleanup ();
+  gegl_compression_cleanup ();
   gegl_random_cleanup ();
+  gegl_parallel_cleanup ();
+  gegl_buffer_swap_cleanup ();
   gegl_cl_cleanup ();
 
   gegl_temp_buffer_free ();
@@ -524,44 +439,10 @@ gegl_exit (void)
 #endif
     }
 
-  if (gegl_swap_dir ())
-    {
-      /* remove all files matching <$GEGL_SWAP>/GEGL-<pid>-*.swap */
-
-      guint         pid     = getpid ();
-      GDir         *dir     = g_dir_open (gegl_swap_dir (), 0, NULL);
-
-      gchar        *glob    = g_strdup_printf ("%i-*", pid);
-      GPatternSpec *pattern = g_pattern_spec_new (glob);
-      g_free (glob);
-
-      if (dir != NULL)
-        {
-          const gchar *name;
-
-          while ((name = g_dir_read_name (dir)) != NULL)
-            {
-              if (g_pattern_match_string (pattern, name))
-                {
-                  gchar *fname = g_build_filename (gegl_swap_dir (),
-                                                   name,
-                                                   NULL);
-                  g_unlink (fname);
-                  g_free (fname);
-                }
-            }
-
-          g_dir_close (dir);
-        }
-
-      g_pattern_spec_free (pattern);
-    }
   g_clear_object (&config);
   global_time = 0;
 }
 
-static void swap_clean (void);
-
 void
 gegl_get_version (int *major,
                   int *minor,
@@ -671,19 +552,32 @@ gegl_post_parse_hook (GOptionContext *context,
 
   if (cmd_gegl_swap)
     g_object_set (config, "swap", cmd_gegl_swap, NULL);
+  if (cmd_gegl_swap_compression)
+    g_object_set (config, "swap-compression", cmd_gegl_swap_compression, NULL);
   if (cmd_gegl_quality)
     config->quality = atof (cmd_gegl_quality);
   if (cmd_gegl_cache_size)
-    config->tile_cache_size = atoll (cmd_gegl_cache_size)*1024*1024;
+    {
+      g_object_set (config,
+                    "tile-cache-size",
+                    (guint64) atoll (cmd_gegl_cache_size) * 1024 * 1024,
+                    NULL);
+    }
   if (cmd_gegl_chunk_size)
     config->chunk_size = atoi (cmd_gegl_chunk_size);
   if (cmd_gegl_tile_size)
     {
       const gchar *str = cmd_gegl_tile_size;
-      config->tile_width = atoi(str);
+      gint         width;
+      gint         height;
+      width = height = atoi(str);
       str = strchr (str, 'x');
       if (str)
-        config->tile_height = atoi(str+1);
+        height = atoi(str+1);
+      g_object_set (config,
+                    "tile-width",  width,
+                    "tile-height", height,
+                    NULL);
     }
   if (cmd_gegl_threads)
     {
@@ -698,10 +592,11 @@ gegl_post_parse_hook (GOptionContext *context,
   if (cmd_gegl_disable_opencl)
     gegl_cl_hard_disable ();
 
-  gegl_init_swap_dir ();
-
   GEGL_INSTRUMENT_START();
 
+  gegl_buffer_swap_init ();
+  gegl_parallel_init ();
+  gegl_compression_init ();
   gegl_operation_gtype_init ();
   gegl_tile_cache_init ();
 
@@ -717,8 +612,6 @@ gegl_post_parse_hook (GOptionContext *context,
 
   gegl_instrument ("gegl", "gegl_init", gegl_ticks () - global_time);
 
-  swap_clean ();
-
   g_signal_connect (G_OBJECT (config),
                    "notify::use-opencl",
                    G_CALLBACK (gegl_config_use_opencl_notify),
