@@ -111,44 +111,37 @@ typedef struct ThreadData
   GeglOperationComposer3Class *klass;
   GeglOperation               *operation;
   GeglOperationContext        *context;
+  GeglBuffer                  *input;
   GeglBuffer                  *aux;
   GeglBuffer                  *aux2;
   GeglBuffer                  *output;
-  gint                        *pending;
+  const GeglRectangle         *roi;
   gint                         level;
   gboolean                     success;
-  GeglRectangle                roi;
 } ThreadData;
 
-static void thread_process (gpointer thread_data, gpointer input)
+static void
+thread_process (const GeglRectangle *area,
+                ThreadData          *data)
 {
-  ThreadData *data = thread_data;
+  GeglBuffer *input;
 
-  if (! input)
+  if (area->x == data->roi->x && area->y == data->roi->y)
+    {
+      input = g_object_ref (data->input);
+    }
+  else
     {
       input = gegl_operation_context_dup_input_maybe_copy (data->context,
-                                                           "input", &data->roi);
+                                                           "input", area);
     }
 
   if (!data->klass->process (data->operation,
                              input, data->aux, data->aux2, 
-                             data->output, &data->roi, data->level))
+                             data->output, area, data->level))
     data->success = FALSE;
 
   g_object_unref (input);
-
-  g_atomic_int_add (data->pending, -1);
-}
-
-static GThreadPool *thread_pool (void)
-{
-  static GThreadPool *pool = NULL;
-  if (!pool)
-  {
-    pool =  g_thread_pool_new (thread_process, NULL, gegl_config_threads (),
-        FALSE, NULL);
-  }
-  return pool;
 }
 
 
@@ -196,55 +189,27 @@ gegl_operation_composer3_process (GeglOperation        *operation,
     {
       if (gegl_operation_use_threading (operation, result))
       {
-        gint threads = gegl_config_threads ();
-        GThreadPool *pool = thread_pool ();
-        ThreadData thread_data[GEGL_MAX_THREADS];
-        gint pending = threads;
+        ThreadData data;
 
-        if (result->width > result->height)
-        {
-          gint bit = result->width / threads;
-          for (gint j = 0; j < threads; j++)
-          {
-            thread_data[j].roi.y = result->y;
-            thread_data[j].roi.height = result->height;
-            thread_data[j].roi.x = result->x + bit * j;
-            thread_data[j].roi.width = bit;
-          }
-          thread_data[threads-1].roi.width = result->width - (bit * (threads-1));
-        }
-        else
-        {
-          gint bit = result->height / threads;
-          for (gint j = 0; j < threads; j++)
-          {
-            thread_data[j].roi.x = result->x;
-            thread_data[j].roi.width = result->width;
-            thread_data[j].roi.y = result->y + bit * j;
-            thread_data[j].roi.height = bit;
-          }
-          thread_data[threads-1].roi.height = result->height - (bit * (threads-1));
-        }
-        for (gint i = 0; i < threads; i++)
-        {
-          thread_data[i].klass = klass;
-          thread_data[i].operation = operation;
-          thread_data[i].context = context;
-          thread_data[i].aux = aux;
-          thread_data[i].aux2 = aux2;
-          thread_data[i].output = output;
-          thread_data[i].pending = &pending;
-          thread_data[i].level = level;
-          thread_data[i].success = TRUE;
-        }
+        data.klass = klass;
+        data.operation = operation;
+        data.context = context;
+        data.input = input;
+        data.aux = aux;
+        data.aux2 = aux2;
+        data.output = output;
+        data.roi = result;
+        data.level = level;
+        data.success = TRUE;
 
-        for (gint i = 1; i < threads; i++)
-          g_thread_pool_push (pool, &thread_data[i], NULL);
-        thread_process (&thread_data[0], g_object_ref (input));
-
-        while (g_atomic_int_get (&pending)) {};
+        gegl_parallel_distribute_area (
+          result,
+          gegl_operation_get_pixels_per_thread (operation),
+          GEGL_SPLIT_STRATEGY_AUTO,
+          (GeglParallelDistributeAreaFunc) thread_process,
+          &data);
         
-        success = thread_data[0].success;
+        success = data.success;
       }
       else
       {
