@@ -22,7 +22,6 @@ void gegl_op_convolution_matrix_register_type(GTypeModule *module);
 void gegl_op_copy_buffer_register_type(GTypeModule *module);
 void gegl_op_difference_of_gaussians_register_type(GTypeModule *module);
 void gegl_op_display_register_type(GTypeModule *module);
-void gegl_op_distance_transform_register_type(GTypeModule *module);
 void gegl_op_dropshadow_register_type(GTypeModule *module);
 void gegl_op_edge_neon_register_type(GTypeModule *module);
 void gegl_op_edge_sobel_register_type(GTypeModule *module);
@@ -104,7 +103,6 @@ void gegl_op_unsharp_mask_register_type(GTypeModule *module);
 void gegl_op_unpremultiply_register_type(GTypeModule *module);
 void gegl_op_value_invert_register_type(GTypeModule *module);
 void gegl_op_vignette_register_type(GTypeModule *module);
-void gegl_op_warp_register_type(GTypeModule *module);
 void gegl_op_waterpixels_register_type(GTypeModule *module);
 void gegl_op_watershed_transform_register_type(GTypeModule *module);
 void gegl_op_wavelet_blur_1d_register_type(GTypeModule *module);
@@ -149,7 +147,6 @@ gegl_module_register (GTypeModule *module)
   gegl_op_copy_buffer_register_type(module);
   gegl_op_difference_of_gaussians_register_type(module);
   gegl_op_display_register_type(module);
-  gegl_op_distance_transform_register_type(module);
   gegl_op_dropshadow_register_type(module);
   gegl_op_edge_neon_register_type(module);
   gegl_op_edge_sobel_register_type(module);
@@ -231,7 +228,6 @@ gegl_module_register (GTypeModule *module)
   gegl_op_unpremultiply_register_type(module);
   gegl_op_value_invert_register_type(module);
   gegl_op_vignette_register_type(module);
-  gegl_op_warp_register_type(module);
   gegl_op_waterpixels_register_type(module);
   gegl_op_watershed_transform_register_type(module);
   gegl_op_wavelet_blur_1d_register_type(module);
