@@ -17,7 +17,6 @@
  * Copyright 2013 Téo Mazars   <teo.mazars@ensimag.fr>
  */
 
-#define GEGL_ITERATOR2_API
 #include "config.h"
 #include <glib/gi18n-lib.h>
 #include <math.h>
@@ -50,6 +49,20 @@ property_int    (size_y, _("Block height"), 16)
     ui_meta     ("unit", "pixel-distance")
     ui_meta     ("axis", "y")
 
+property_int    (offset_x, _("Offset X"), 0)
+    description (_("Horizontal offset of blocks in pixels"))
+    value_range (G_MININT, G_MAXINT)
+    ui_range    (0, 2048)
+    ui_meta     ("unit", "pixel-coordinate")
+    ui_meta     ("axis", "x")
+
+property_int    (offset_y, _("Offset Y"), 0)
+    description (_("Vertical offset of blocks in pixels"))
+    value_range (G_MININT, G_MAXINT)
+    ui_range    (0, 2048)
+    ui_meta     ("unit", "pixel-coordinate")
+    ui_meta     ("axis", "y")
+
 property_double (ratio_x, _("Size ratio X"), 1.0)
     description (_("Horizontal size ratio of a pixel inside each block"))
     value_range (0.0, 1.0)
@@ -76,6 +89,15 @@ property_color  (background, _("Background color"), "white")
 #define ALLOC_THRESHOLD_SIZE (64)
 #define SQR(x)               ((x)*(x))
 
+static gint
+align_offset (gint offset,
+              gint size)
+{
+  gint align = abs(offset) % size;
+  
+  return offset <= 0 ? align : (size - align);
+}
+
 static void
 prepare (GeglOperation *operation)
 {
@@ -297,8 +319,11 @@ pixelize_noalloc (GeglBuffer          *input,
                   GeglProperties      *o,
                   const Babl          *format)
 {
-  gint start_x = block_index (roi->x, o->size_x) * o->size_x;
-  gint start_y = block_index (roi->y, o->size_y) * o->size_y;
+  gint align_x = align_offset(o->offset_x, o->size_x);
+  gint align_y = align_offset(o->offset_y, o->size_y);
+
+  gint start_x = block_index (roi->x, o->size_x) * o->size_x - align_x;
+  gint start_y = block_index (roi->y, o->size_y) * o->size_y - align_y;
   gint x, y;
   gint off_shape_x, off_shape_y;
 
@@ -343,8 +368,12 @@ pixelize (gfloat              *input,
           const GeglRectangle *whole_region,
           GeglProperties          *o)
 {
-  gint          start_x = block_index (roi->x, o->size_x) * o->size_x;
-  gint          start_y = block_index (roi->y, o->size_y) * o->size_y;
+  
+  gint          align_x = align_offset(o->offset_x, o->size_x);
+  gint          align_y = align_offset(o->offset_y, o->size_y);
+
+  gint          start_x = block_index (roi->x, o->size_x) * o->size_x - align_x;
+  gint          start_y = block_index (roi->y, o->size_y) * o->size_y - align_y;
   gint          x, y;
   gint          off_shape_x, off_shape_y;
   gfloat        color[4];
@@ -367,18 +396,18 @@ pixelize (gfloat              *input,
 
         gegl_rectangle_intersect (&rect, whole_region, &rect);
 
-        if (rect.width < 1 || rect.height < 1)
-          continue;
-
         rect2.x = rect.x - extended_roi->x;
         rect2.y = rect.y - extended_roi->y;
         rect2.width  = rect.width;
         rect2.height = rect.height;
 
-        mean_rectangle (input, &rect2, extended_roi->width, color);
-
         gegl_rectangle_intersect (&rect, roi, &rect);
 
+        if (rect.width < 1 || rect.height < 1)
+          continue;
+
+        mean_rectangle (input, &rect2, extended_roi->width, color);
+
         rect2.x = rect.x - roi->x;
         rect2.y = rect.y - roi->y;
         rect2.width  = rect.width;
