@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2003,2004,2007, 2015 Øyvind Kolås <pippin@gimp.org>
  */
@@ -88,6 +88,13 @@ property_int (me_subpel_quality, _("me-subpel-quality"), 0)
 #include <libavutil/opt.h>
 #include <libswscale/swscale.h>
 
+/* remove if libavcodec_required_version is changed to > 56.41.100 */
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(56,41,100)
+# define AV_CODEC_FLAG_GLOBAL_HEADER	CODEC_FLAG_GLOBAL_HEADER
+# define AV_CODEC_CAP_VARIABLE_FRAME_SIZE	CODEC_CAP_VARIABLE_FRAME_SIZE
+# define AV_CODEC_CAP_INTRA_ONLY	CODEC_CAP_INTRA_ONLY
+#endif
+
 typedef struct
 {
   gdouble    frame;
@@ -290,7 +297,7 @@ add_audio_stream (GeglProperties *o, AVFormatContext * oc, int codec_id)
   c->codec_type = AVMEDIA_TYPE_AUDIO;
 
   if (oc->oformat->flags & AVFMT_GLOBALHEADER)
-    c->flags |= CODEC_FLAG_GLOBAL_HEADER;
+    c->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
 
   return st;
 }
@@ -631,7 +638,7 @@ add_video_stream (GeglProperties *o, AVFormatContext * oc, int codec_id)
 #endif
 
    if (oc->oformat->flags & AVFMT_GLOBALHEADER)
-     c->flags |= CODEC_FLAG_GLOBAL_HEADER;
+     c->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
 
   return st;
 }
@@ -699,7 +706,9 @@ open_video (GeglProperties *o, AVFormatContext * oc, AVStream * st)
     }
 
   p->video_outbuf = NULL;
+#if (LIBAVFORMAT_VERSION_MAJOR < 58) /* AVFMT_RAWPICTURE got removed from ffmpeg: "not used anymore" */
   if (!(oc->oformat->flags & AVFMT_RAWPICTURE))
+#endif
     {
       /* allocate output buffer, 1 mb / frame, might fail for some codecs on UHD - but works for now */
       p->video_outbuf_size = 1024 * 1024;
@@ -803,6 +812,7 @@ write_video_frame (GeglProperties *o,
   picture_ptr      = p->picture;
   picture_ptr->pts = p->frame_count;
 
+	#if (LIBAVFORMAT_VERSION_MAJOR < 58) /* AVFMT_RAWPICTURE got removed from ffmpeg: "not used anymore" */
   if (oc->oformat->flags & AVFMT_RAWPICTURE)
     {
       /* raw video case. The API will change slightly in the near
@@ -821,6 +831,7 @@ write_video_frame (GeglProperties *o,
       ret = av_write_frame (oc, &pkt);
     }
   else
+#endif
     {
       /* encode the image */
       AVPacket pkt2;
