@@ -14,7 +14,7 @@
  * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Philip Lafleur
- *           2006-2012 Øyvind Kolås
+ *           2006-2018 Øyvind Kolås
  *           2009 Martin Nordholts
  *           2010 Debarshi Ray
  *           2011 Mikael Magnusson
@@ -24,6 +24,7 @@
  *           2012 Nicolas Robidoux
  */
 
+
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
@@ -31,7 +32,7 @@
 #include <gegl.h>
 #include <gegl-plugin.h>
 
-#include "buffer/gegl-buffer-cl-cache.h"
+#include "opencl/gegl-buffer-cl-cache.h"
 
 #include "gegl-config.h"
 
@@ -97,7 +98,6 @@ static GeglNode     *gegl_transform_detect                       (GeglOperation
                                                                   gint                  x,
                                                                   gint                  y);
 
-static gboolean      gegl_matrix3_is_affine                      (GeglMatrix3          *matrix);
 static gboolean      gegl_transform_matrix3_allow_fast_translate (GeglMatrix3          *matrix);
 static void          gegl_transform_create_composite_matrix      (OpTransform          *transform,
                                                                   GeglMatrix3          *matrix);
@@ -147,10 +147,11 @@ op_transform_get_type (void)
 static void
 gegl_transform_prepare (GeglOperation *operation)
 {
-  const Babl *space = gegl_operation_get_source_space (operation, "input");
-  const Babl *format = babl_format_with_space ("RaGaBaA float", space);
+  const Babl *source_format = gegl_operation_get_source_format (operation, "input");
+  const Babl *space = source_format?babl_format_get_space (source_format):NULL;
   GeglMatrix3  matrix;
   OpTransform *transform = (OpTransform *) operation;
+  const Babl *format = source_format;
 
   gegl_transform_create_composite_matrix (transform, &matrix);
 
@@ -158,13 +159,23 @@ gegl_transform_prepare (GeglOperation *operation)
   if (gegl_transform_is_intermediate_node (transform) ||
       gegl_transform_matrix3_allow_fast_translate (&matrix) ||
       (gegl_matrix3_is_translate (&matrix) &&
-       transform->sampler == GEGL_SAMPLER_NEAREST) ||
-      (transform->sampler == GEGL_SAMPLER_NEAREST))
+       transform->sampler == GEGL_SAMPLER_NEAREST))
     {
-      const Babl *fmt = gegl_operation_get_source_format (operation, "input");
-
-      if (fmt)
-        format = fmt;
+    }
+  else if (transform->sampler == GEGL_SAMPLER_NEAREST)
+    {
+      if (source_format && ! babl_format_has_alpha (source_format))
+        format = gegl_babl_variant (source_format, GEGL_BABL_VARIANT_ALPHA);
+    }
+  else
+    {
+      BablModelFlag model_flags = babl_get_model_flags (source_format);
+      if (model_flags & BABL_MODEL_FLAG_CMYK)
+        format = babl_format_with_space ("camayakaA float", space);
+      else if (model_flags & BABL_MODEL_FLAG_GRAY)
+        format = babl_format_with_space ("YaA float", space);
+      else
+        format = babl_format_with_space ("RaGaBaA float", space);
     }
 
   gegl_operation_set_format (operation, "input", format);
@@ -187,7 +198,6 @@ op_transform_class_init (OpTransformClass *klass)
   op_class->detect                    = gegl_transform_detect;
   op_class->process                   = gegl_transform_process;
   op_class->prepare                   = gegl_transform_prepare;
-  op_class->no_cache                  = TRUE;
   op_class->threaded                  = TRUE;
 
   klass->create_matrix                = NULL;
@@ -823,7 +833,7 @@ gegl_transform_detect (GeglOperation *operation,
       gegl_matrix3_is_identity (&inverse))
     return gegl_operation_detect (source, x, y);
 
-  gegl_transform_create_matrix (transform, &inverse);
+  gegl_transform_create_composite_matrix (transform, &inverse);
   gegl_matrix3_invert (&inverse);
 
   /*
@@ -1060,47 +1070,39 @@ typedef struct ThreadData
                 gint                 level);
 
 
-  GeglOperation            *operation;
-  GeglOperationContext     *context;
-  GeglBuffer               *output;
-  gint                     *pending;
-  GeglMatrix3              *matrix;
-  gint                      level;
-  gboolean                  success;
-  GeglRectangle             roi;
+  GeglOperation        *operation;
+  GeglOperationContext *context;
+  GeglBuffer           *input;
+  GeglBuffer           *output;
+  GeglMatrix3          *matrix;
+  const GeglRectangle  *roi;
+  gint                  level;
 } ThreadData;
 
-static void thread_process (gpointer thread_data, gpointer input)
+static void
+thread_process (const GeglRectangle *area,
+                ThreadData          *data)
 {
-  ThreadData *data = thread_data;
+  GeglBuffer *input;
 
-  if (! input)
+  if (area->x == data->roi->x && area->y == data->roi->y)
+    {
+      input = g_object_ref (data->input);
+    }
+  else
     {
       input = gegl_operation_context_dup_input_maybe_copy (data->context,
-                                                           "input", &data->roi);
+                                                           "input", area);
     }
 
   data->func (data->operation,
               data->output,
               input,
               data->matrix,
-              &data->roi,
+              area,
               data->level);
 
   g_object_unref (input);
-
-  g_atomic_int_add (data->pending, -1);
-}
-
-static GThreadPool *thread_pool (void)
-{
-  static GThreadPool *pool = NULL;
-  if (!pool)
-    {
-      pool =  g_thread_pool_new (thread_process, NULL, gegl_config_threads(),
-                                 FALSE, NULL);
-    }
-  return pool;
 }
 
 
@@ -1112,13 +1114,12 @@ transform_affine (GeglOperation       *operation,
                   const GeglRectangle *roi,
                   gint                 level)
 {
-  const Babl *space = gegl_operation_get_source_space (operation, "input");
   gint             factor = 1 << level;
   OpTransform     *transform = (OpTransform *) operation;
-  const Babl      *format = babl_format_with_space ("RaGaBaA float", space);
+  const Babl      *format = gegl_operation_get_format (operation, "output");
   GeglMatrix3      inverse;
   gdouble          inverse_near_z = 1.0 / transform->near_z;
-  GeglMatrix2      inverse_jacobian;
+  GeglBufferMatrix2 inverse_jacobian;
   GeglAbyssPolicy  abyss_policy = gegl_transform_get_abyss_policy (transform);
   GeglSampler     *sampler = gegl_buffer_sampler_new_at_level (src,
                                          format,
@@ -1130,6 +1131,7 @@ transform_affine (GeglOperation       *operation,
   GeglRectangle  bounding_box = *gegl_buffer_get_abyss (src);
   GeglRectangle  context_rect = *gegl_sampler_get_context_rect (sampler);
   GeglRectangle  dest_extent  = *roi;
+  gint           components = babl_format_get_n_components (format);
 
   bounding_box.x      += context_rect.x;
   bounding_box.y      += context_rect.y;
@@ -1173,7 +1175,7 @@ transform_affine (GeglOperation       *operation,
                                                       level,
                                                       format,
                                                       GEGL_ACCESS_WRITE,
-                                                      GEGL_ABYSS_NONE);
+                                                      GEGL_ABYSS_NONE, 1);
 
     /*
      * Hoist most of what can out of the while loop:
@@ -1196,8 +1198,8 @@ transform_affine (GeglOperation       *operation,
 
     while (gegl_buffer_iterator_next (i))
       {
-        GeglRectangle *roi = &i->roi[0];
-        gfloat * restrict dest_ptr = (gfloat *)i->data[0];
+        GeglRectangle *roi = &i->items[0].roi;
+        gfloat * restrict dest_ptr = (gfloat *)i->items[0].data;
 
         gdouble u_start =
           base_u +
@@ -1223,8 +1225,8 @@ transform_affine (GeglOperation       *operation,
 
               gint x;
 
-              memset (dest_ptr, 0, (gint) 4 * sizeof (gfloat) * x1);
-              dest_ptr += (gint) 4 * x1;
+              memset (dest_ptr, 0, (gint) components * sizeof (gfloat) * x1);
+              dest_ptr += (gint) components * x1;
 
               u_float += x1 * inverse_jacobian.coeff [0][0];
               v_float += x1 * inverse_jacobian.coeff [1][0];
@@ -1236,19 +1238,19 @@ transform_affine (GeglOperation       *operation,
                                    &inverse_jacobian,
                                    dest_ptr,
                                    abyss_policy);
-                  dest_ptr += (gint) 4;
+                  dest_ptr += (gint) components;
 
                   u_float += inverse_jacobian.coeff [0][0];
                   v_float += inverse_jacobian.coeff [1][0];
                 }
 
-              memset (dest_ptr, 0, (gint) 4 * sizeof (gfloat) * (roi->width - x2));
-              dest_ptr += (gint) 4 * (roi->width - x2);
+              memset (dest_ptr, 0, (gint) components * sizeof (gfloat) * (roi->width - x2));
+              dest_ptr += (gint) components * (roi->width - x2);
             }
           else
             {
-              memset (dest_ptr, 0, (gint) 4 * sizeof (gfloat) * roi->width);
-              dest_ptr += (gint) 4 * roi->width;
+              memset (dest_ptr, 0, (gint) components * sizeof (gfloat) * roi->width);
+              dest_ptr += (gint) components * roi->width;
             }
 
           u_start += inverse_jacobian.coeff [0][1];
@@ -1268,9 +1270,8 @@ transform_generic (GeglOperation       *operation,
                    const GeglRectangle *roi,
                    gint                 level)
 {
-  const Babl *space = gegl_operation_get_source_space (operation, "input");
   OpTransform *transform = (OpTransform *) operation;
-  const Babl          *format = babl_format_with_space ("RaGaBaA float", space);
+  const Babl          *format = gegl_operation_get_format (operation, "output");
   gint                 factor = 1 << level;
   GeglBufferIterator  *i;
   GeglMatrix3          inverse;
@@ -1285,6 +1286,7 @@ transform_generic (GeglOperation       *operation,
   GeglRectangle  bounding_box = *gegl_buffer_get_abyss (src);
   GeglRectangle  context_rect = *gegl_sampler_get_context_rect (sampler);
   GeglRectangle  dest_extent  = *roi;
+  gint           components   = babl_format_get_n_components (format);
 
   bounding_box.x      += context_rect.x;
   bounding_box.y      += context_rect.y;
@@ -1304,7 +1306,7 @@ transform_generic (GeglOperation       *operation,
                                 level,
                                 format,
                                 GEGL_ACCESS_WRITE,
-                                GEGL_ABYSS_NONE);
+                                GEGL_ABYSS_NONE, 1);
 
   gegl_matrix3_copy_into (&inverse, matrix);
 
@@ -1325,7 +1327,7 @@ transform_generic (GeglOperation       *operation,
    */
   while (gegl_buffer_iterator_next (i))
     {
-      GeglRectangle *roi = &i->roi[0];
+      GeglRectangle *roi = &i->items[0].roi;
 
       gdouble u_start =
         inverse.coeff [0][0] * (roi->x + (gdouble) 0.5) +
@@ -1340,7 +1342,7 @@ transform_generic (GeglOperation       *operation,
         inverse.coeff [2][1] * (roi->y + (gdouble) 0.5) +
         inverse.coeff [2][2];
 
-      gfloat * restrict dest_ptr = (gfloat *) i->data[0];
+      gfloat * restrict dest_ptr = (gfloat *)i->items[0].data;
 
       /*
        * Assumes that height and width are > 0.
@@ -1361,8 +1363,8 @@ transform_generic (GeglOperation       *operation,
 
             gint x;
 
-            memset (dest_ptr, 0, (gint) 4 * sizeof (gfloat) * x1);
-            dest_ptr += (gint) 4 * x1;
+            memset (dest_ptr, 0, (gint) components * sizeof (gfloat) * x1);
+            dest_ptr += (gint) components * x1;
 
             u_float += x1 * inverse.coeff [0][0];
             v_float += x1 * inverse.coeff [1][0];
@@ -1374,7 +1376,7 @@ transform_generic (GeglOperation       *operation,
                 gdouble u = u_float * w_recip;
                 gdouble v = v_float * w_recip;
 
-                GeglMatrix2 inverse_jacobian;
+                GeglBufferMatrix2 inverse_jacobian;
                 inverse_jacobian.coeff [0][0] =
                   (inverse.coeff [0][0] - inverse.coeff [2][0] * u) * w_recip;
                 inverse_jacobian.coeff [0][1] =
@@ -1390,19 +1392,19 @@ transform_generic (GeglOperation       *operation,
                                  dest_ptr,
                                  abyss_policy);
 
-                dest_ptr += (gint) 4;
+                dest_ptr += (gint) components;
                 u_float += inverse.coeff [0][0];
                 v_float += inverse.coeff [1][0];
                 w_float += inverse.coeff [2][0];
               }
 
-            memset (dest_ptr, 0, (gint) 4 * sizeof (gfloat) * (roi->width - x2));
-            dest_ptr += (gint) 4 * (roi->width - x2);
+            memset (dest_ptr, 0, (gint) components * sizeof (gfloat) * (roi->width - x2));
+            dest_ptr += (gint) components * (roi->width - x2);
           }
         else
           {
-            memset (dest_ptr, 0, (gint) 4 * sizeof (gfloat) * roi->width);
-            dest_ptr += (gint) 4 * roi->width;
+            memset (dest_ptr, 0, (gint) components * sizeof (gfloat) * roi->width);
+            dest_ptr += (gint) components * roi->width;
           }
 
         u_start += inverse.coeff [0][1];
@@ -1451,7 +1453,7 @@ transform_nearest (GeglOperation       *operation,
                                 level,
                                 format,
                                 GEGL_ACCESS_WRITE,
-                                GEGL_ABYSS_NONE);
+                                GEGL_ABYSS_NONE, 1);
 
   gegl_matrix3_copy_into (&inverse, matrix);
 
@@ -1472,7 +1474,7 @@ transform_nearest (GeglOperation       *operation,
    */
   while (gegl_buffer_iterator_next (i))
     {
-      GeglRectangle *roi = &i->roi[0];
+      GeglRectangle *roi = &i->items[0].roi;
 
       gdouble u_start =
         inverse.coeff [0][0] * (roi->x + (gdouble) 0.5) +
@@ -1487,7 +1489,7 @@ transform_nearest (GeglOperation       *operation,
         inverse.coeff [2][1] * (roi->y + (gdouble) 0.5) +
         inverse.coeff [2][2];
 
-      guchar * restrict dest_ptr = (guchar *) i->data[0];
+      guchar * restrict dest_ptr = (guchar *) i->items[0].data;
 
       /*
        * Assumes that height and width are > 0.
@@ -1558,18 +1560,6 @@ static inline gboolean is_zero (const gdouble f)
           GEGL_TRANSFORM_CORE_EPSILON*GEGL_TRANSFORM_CORE_EPSILON);
 }
 
-static inline gboolean is_one (const gdouble f)
-{
-  return (is_zero (f-(gdouble) 1.0));
-}
-
-static gboolean gegl_matrix3_is_affine (GeglMatrix3 *matrix)
-{
-  return (is_zero (matrix->coeff [2][0]) &&
-          is_zero (matrix->coeff [2][1]) &&
-          is_one  (matrix->coeff [2][2]));
-}
-
 static gboolean
 gegl_transform_matrix3_allow_fast_translate (GeglMatrix3 *matrix)
 {
@@ -1651,6 +1641,9 @@ gegl_transform_process (GeglOperation        *operation,
     }
   else
     {
+      gboolean is_cmyk =
+       ((babl_get_model_flags (gegl_operation_get_format (operation, "output"))
+           & BABL_MODEL_FLAG_CMYK) != 0);
       /*
        * For other cases, do a proper resampling
        */
@@ -1661,7 +1654,10 @@ gegl_transform_process (GeglOperation        *operation,
                     const GeglRectangle *roi,
                     gint                 level) = transform_generic;
 
-      if (gegl_matrix3_is_affine (&matrix))
+      /* XXX; why does the affine code path mangle CMYK colors when
+              the generic one does not?
+       */
+      if (gegl_matrix3_is_affine (&matrix) && !is_cmyk)
         func = transform_affine;
 
       if (transform->sampler == GEGL_SAMPLER_NEAREST)
@@ -1672,58 +1668,27 @@ gegl_transform_process (GeglOperation        *operation,
 
       /* flush opencl caches, to avoid racy flushing
        */
-      if (gegl_cl_is_accelerated ())
-        gegl_buffer_cl_cache_flush (input, NULL);
+      gegl_buffer_flush_ext (input, NULL);
 
       if (gegl_operation_use_threading (operation, result))
       {
-        gint threads = gegl_config_threads ();
-        GThreadPool *pool = thread_pool ();
-        ThreadData thread_data[GEGL_MAX_THREADS];
-        gint pending = threads;
+        ThreadData data;
 
-        if (result->width > result->height)
-        {
-          gint bit = result->width / threads;
-          for (gint j = 0; j < threads; j++)
-          {
-            thread_data[j].roi.y = result->y;
-            thread_data[j].roi.height = result->height;
-            thread_data[j].roi.x = result->x + bit * j;
-            thread_data[j].roi.width = bit;
-          }
-          thread_data[threads-1].roi.width = result->width - (bit * (threads-1));
-        }
-        else
-        {
-          gint bit = result->height / threads;
-          for (gint j = 0; j < threads; j++)
-          {
-            thread_data[j].roi.x = result->x;
-            thread_data[j].roi.width = result->width;
-            thread_data[j].roi.y = result->y + bit * j;
-            thread_data[j].roi.height = bit;
-          }
-          thread_data[threads-1].roi.height = result->height - (bit * (threads-1));
-        }
+        data.func = func;
+        data.matrix = &matrix;
+        data.operation = operation;
+        data.context = context;
+        data.input = input;
+        data.output = output;
+        data.roi = result;
+        data.level = level;
 
-        for (gint i = 0; i < threads; i++)
-        {
-          thread_data[i].func = func;
-          thread_data[i].matrix = &matrix;
-          thread_data[i].operation = operation;
-          thread_data[i].context = context;
-          thread_data[i].output = output;
-          thread_data[i].pending = &pending;
-          thread_data[i].level = level;
-          thread_data[i].success = TRUE;
-        }
-
-        for (gint i = 1; i < threads; i++)
-          g_thread_pool_push (pool, &thread_data[i], NULL);
-        thread_process (&thread_data[0], g_object_ref (input));
-
-        while (g_atomic_int_get (&pending)) {};
+        gegl_parallel_distribute_area (
+          result,
+          gegl_operation_get_pixels_per_thread (operation),
+          GEGL_SPLIT_STRATEGY_AUTO,
+          (GeglParallelDistributeAreaFunc) thread_process,
+          &data);
       }
       else
       {
